/* Mixture of:
 *
 * Ozon OS Dock
 * Panel OSD
 * Disable message tray
 * Show message tray
 */

const St = imports.gi.St;
const Shell = imports.gi.Shell;
const Main = imports.ui.main;
const LayoutManager = Main.layoutManager;
const MessageTray = Main.messageTray;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Gio = imports.gi.Gio;
const GnomeDash = Me.imports.gnomedash;
const Intellihide = Me.imports.intellihide;
const AtomDock = Me.imports.atomdock;

// Disable message tray
let messageTrayButtonSignal = null;
let messageTrayDwell = 0;
let messageTrayButton;

let oldDash;
let atomDock;
let intellihide;
let settings = null;
let settingsTraySignal = null;
let settingsIconSignal = null;
let settingsDisableMessageTray = true;

// Settings
const SETTINGS_DISABLE_MESSAGE_TRAY = "disable-message-tray";
const SETTINGS_MAX_ICON_SIZE = "max-icon-size";

// Show message tray 'top bar' button
function showMessageTray() {
	if (MessageTray._trayState === 0) {
		MessageTray.openTray();
	} else {
		MessageTray.hide();
	}
}

// Disable message tray
function disableMessageTray() {

    messageTrayDwell = MessageTray._trayDwellTimeout;

    if (settingsDisableMessageTray) {
        if("_trayPressure" in LayoutManager) {
            LayoutManager._trayPressure._keybindingMode =
                Shell.KeyBindingMode.OVERVIEW;
        }
        
        MessageTray._trayDwellTimeout = function() { return false; };
    }

	Main.panel._rightBox.insert_child_at_index(messageTrayButton, 0);
}

// Enable message tray
function enableMessageTray() {

    if("_trayPressure" in LayoutManager) {
        LayoutManager._trayPressure._keybindingMode = 
            Shell.KeyBindingMode.NORMAL | Shell.KeyBindingMode.OVERVIEW;
    }

    if(messageTrayDwell) {
        MessageTray._trayDwellTimeout = messageTrayDwell;
        messageTrayDwell = 0;
    }

	Main.panel._rightBox.remove_child(messageTrayButton);
}

function settingsTrayChanged() {
    settingsDisableMessageTray = settings.get_boolean(SETTINGS_DISABLE_MESSAGE_TRAY);
    if (settingsDisableMessageTray) {
        disableMessageTray();
    } else {
        enableMessageTray();
    }
}

function settingsIconChanged() {
    atomDock.setMaxIconSize(settings.get_int(SETTINGS_MAX_ICON_SIZE), true);
}

// Init settings
function initSettings() {
    const GioSSS = Gio.SettingsSchemaSource;
    let schemaSource = GioSSS.new_from_directory(Me.path + "/schemas", 
            GioSSS.get_default(), false);

    let schemaObj = schemaSource.lookup(Me.metadata["settings-schema"], true);
    if(!schemaObj) {
        throw new Error("Schema " + Me.metadata["settings-schema"] + " could not be found for extension " +
                        Me.uuid + ". Please check your installation.");
    }
    settings = new Gio.Settings({ settings_schema: schemaObj });
    settingsDisableMessageTray = settings.get_boolean(SETTINGS_DISABLE_MESSAGE_TRAY);
}

// Init message tray button
function initMessageTrayButton() {
    messageTrayButton = new St.Bin({ style_class: 'panel-button',
                          reactive: true,
                          can_focus: true,
                          x_fill: true,
                          y_fill: false,
                          track_hover: true });
    let icon = new St.Icon({ icon_name: 'preferences-system-notifications-symbolic',
                             style_class: 'system-status-icon' });

    messageTrayButton.set_child(icon);
}

function init() {
    oldDash = new GnomeDash.GnomeDash();

    initSettings();
	initMessageTrayButton();
}

function show() {
    atomDock.disableAutoHide();
}

function hide() {
    atomDock.enableAutoHide();
}

function retop() {
    atomDock.retop();
}

function enable() {
    // Hide old dash
    oldDash.hideDash();

    // Disable message tray
    disableMessageTray();

    // Enable new dock
    atomDock = new AtomDock.AtomDock();
    let iconSize = settings.get_int(SETTINGS_MAX_ICON_SIZE);
    if (iconSize) {
        atomDock.setMaxIconSize(iconSize, false);
    } else {
        atomDock.setMaxIconSize(48, false);
    }

    intellihide = new Intellihide.Intellihide(show, hide, retop, atomDock);

    messageTrayButtonSignal = messageTrayButton.connect('button-release-event', showMessageTray);

    settingsTraySignal = settings.connect("changed::" + SETTINGS_DISABLE_MESSAGE_TRAY,
        function() { settingsTrayChanged(); });

    settingsIconSignal = settings.connect("changed::" + SETTINGS_MAX_ICON_SIZE,
        function() { settingsIconChanged(); });
}

function disable() {
    intellihide.destroy();
    atomDock.destroy();
    oldDash.showDash();
    
    enableMessageTray();
    
    if(messageTrayDwell) {
        MessageTray._trayDwellTimeout = messageTrayDwell;
        messageTrayDwell = 0;
		Main.panel._rightBox.remove_child(messageTrayButton);
    }

	if(messageTrayButtonSignal != null) {
		messageTrayButton.disconnect(messageTrayButtonSignal);
		messageTrayButtonSignal = null;
	}

    if(settingsTraySignal != null) {
        settings.disconnect(settingsTraySignal);
        settingsTraySignal = null;
    }

    if(settingsIconSignal != null) {
        settings.disconnect(settingsIconSignal);
        settingsIconSignal = null;
    }
}
